{% extends "staff/base.html" %}

{% block content %}

<form method="post" class="card">
    {{ csrf_input }}
    <div class="p-4">
        {% block form_content %}{% endblock %}
        <button class="px-5 py-2 mt-6 text-white bg-blue-600 rounded hover:bg-blue-700" type="submit">Save</button>
    </div>
</form>

{% endblock %}
